import java.util.Properties;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ejb.EJBHome;
import javax.ejb.EJBException;
import javax.rmi.PortableRemoteObject;

public class EjbProxy
{
    private Properties _prop = null;

    public EjbProxy()
    {
    }

    public EjbProxy (String initContextFactory, String providerUrl)
    {
        setContextProperties (initContextFactory, providerUrl, null, null);
    }

    public void EjbProxy (Properties prop)
    {
         setContextProperties (prop);
    }

    public void setContextProperties (Properties prop)
    {
        _prop = prop;
    }

    public void setContextProperties (String initContextFactory, String providerUrl,
                                      String user, String password)
    {
        _prop = new Properties ();
        _prop.put (Context.INITIAL_CONTEXT_FACTORY, initContextFactory);
        _prop.put (Context.PROVIDER_URL, providerUrl);
        if (user != null)
        {
        	_prop.put(Context.SECURITY_PRINCIPAL, user);
        	if (password == null)
            		password = "";
        	_prop.put(Context.SECURITY_CREDENTIALS, password);
        }
    }

    public void setContextUserParam (String user, String password)
    {
        if (_prop == null)
            _prop = new Properties ();

        _prop.put(Context.SECURITY_PRINCIPAL, user);
        _prop.put(Context.SECURITY_CREDENTIALS, password);
    }

    public EJBHome getHome (String beanJndiLookupName) throws EJBException
    {
        try
        {
            InitialContext ctx = null;

	        if (_prop != null)
            		ctx = new InitialContext (_prop);
            else
            		ctx = new InitialContext ();

            Object home = ctx.lookup(beanJndiLookupName);
            EJBHome obHome = (EJBHome)PortableRemoteObject.narrow (home, EJBHome.class);
            return obHome;
        }
        catch (NamingException ne)
        {
            throw new EJBException (ne);
        }
    }

    public Object getObj (String beanJndiLookupName) throws EJBException
    {
        try
        {
            EJBHome obHome = getHome (beanJndiLookupName);
            //get the method of create
            Method m = obHome.getClass().getDeclaredMethod("create", new Class[0]);
            //invoke the create method
            Object obj = m.invoke (obHome, new Object[0]);
            return obj;
        }
        catch (NoSuchMethodException ne)
        {
            throw new EJBException (ne);
        }
        catch (InvocationTargetException ie)
        {
            throw new EJBException (ie);
        }
        catch (IllegalAccessException iae)
        {
            throw new EJBException (iae);
        }
    }
}
